use std::sync::Arc;

use axum::extract::State;
use maud::{html, Markup, PreEscaped, DOCTYPE};

use crate::{
    clock,
    data::{self, CombinedFish, Filters},
    AppState,
};

pub fn layout(content: Markup) -> Markup {
    html! {
        (DOCTYPE)
        html {
            head {
                title { "beacon" }
                meta name="viewport" content="width=device-width";
                link rel="stylesheet" href="/static/style.css";
                script src="/static/htmx.js" {}
            }

            body { (content) }
        }
    }
}

pub fn main_page(
    state: State<Arc<AppState>>,
    caught_fish: Vec<i32>,
    acc_id: String,
    filters: &Filters,
) -> Markup {
    let meta = state.data.fish_with_meta();
    let mut values: Vec<&CombinedFish> = filters.filter(meta.values().collect(), &caught_fish);

    values.sort_by(|afish, bfish| {
        bfish
            .is_up
            .cmp(&afish.is_up)
            .then(bfish.is_always_up.cmp(&afish.is_always_up))
            .then(bfish.rarity.total_cmp(&afish.rarity).reverse())
            .then(bfish.meta.name_en.cmp(&afish.meta.name_en))
    });
    // values.sort_by(|afish, bfish| {
    //     if !afish.is_up && !bfish.is_up && !afish.windows.is_empty() && !bfish.windows.is_empty() {
    //         bfish
    //             .windows
    //             .first()
    //             .unwrap()
    //             .start_time
    //             .cmp(&afish.windows.first().unwrap().start_time)
    //             .reverse()
    //     } else {
    //         Ordering::Equal
    //     }
    // });
    let template = html! {
        .header {
            h1 { "Hello! Current ET: " (clock::get_current_eorzea_date().format("%H:%M")) }
            .side {
                details {
                    summary { "Filters" }
                    form {
                        fieldset {
                            @if filters.non_big_fish {
                                input name="big" id="big" type="checkbox" checked;
                            } @else {
                                input name="big" id="big" type="checkbox";
                            }
                            label for="big" { "Include non big fish?" }
                        }

                        fieldset {
                            @if filters.include_caught {
                                input name="caught" id="caught" type="checkbox" checked;
                            } @else {
                                input name="caught" id="caught" type="checkbox";
                            }
                            label for="caught" { "Include caught fish?" }
                        }

                        fieldset {
                            label for="patches" { "Filter by patch" }
                            br;
                            select name="patches" id="patches" multiple {
                                @for patch in data::PATCHES {
                                    @if filters.patches.contains(&patch) {
                                        option value=(patch) selected { (format!("{:.1}", patch)) }
                                    } @else {
                                        option value=(patch) { (format!("{:.1}", patch)) }
                                    }
                                }
                            }
                        }

                        button type="submit" { "Apply" }
                    }
                }
            }
        }
        @for fish in values {
            section.up[fish.is_up].alwaysup[fish.is_always_up] {
                .title {
                    div {
                        @if !caught_fish.contains(&(fish.entry.id as i32)) {
                            form action=(format!("/{}/catch/{}", acc_id, fish.entry.id)) method="post" {
                                button.catch-button type="submit" { (PreEscaped("&check;")) }
                            }
                        }
                    }
                    div {
                        h3 { (fish.meta.name_en) }
                        .subtitle {
                            "Patch " (fish.entry.patch)
                        }
                    }
                }
                .when {
                    @if let Some(window) = fish.windows.first() {
                        @if fish.is_up || fish.is_always_up {
                            "closes " (window.display_end_time())
                        } @else {
                            "opens " (window.display_start_time())
                        }
                    }
                }
                .how {
                    @for item_id in &fish.entry.best_catch_path {
                        @if let Some(item) = state.data.db_data.items.get(item_id) {
                            span.catchpath title=(item.name_en) {
                                img src=(item.get_icon_url()) width="35";

                                @if let Some(hookset) = item.get_hookset(&state.data) {
                                    img.hookset src=(hookset) width="20";
                                }

                                @if let Some(tug) = item.get_tug(&state.data) {
                                    span.tug { (tug) }
                                }
                            }
                        }
                    }
                    @if let Some(hookset) = fish.entry.hookset {
                        span.catchpath {
                            img src=(hookset) width="35";

                            @if let Some(tug) = fish.entry.tug {
                                span.tug { (tug) }
                            }
                        }

                    }
                }
                .meta {
                    @if fish.entry.start_hour.is_some() && fish.entry.end_hour.is_some() {
                        div {
                            @if !fish.is_always_up {
                                (clock::display_eorzea_time(&clock::set_hm_from_float(&clock::get_current_eorzea_date(), fish.entry.start_hour.unwrap())))
                                " to "
                                (clock::display_eorzea_time(&clock::set_hm_from_float(&clock::get_current_eorzea_date(), fish.entry.end_hour.unwrap())))
                            } @else {
                                "always up!"
                            }
                        }
                    }
                    div { "Rarity: " (format!("{:.2}", fish.rarity * 100.)) "%" }
                }
            }
        }
    };

    layout(html! {
        main hx-get="" hx-trigger="every 10s" {
            (template)
        }
    })
}

pub fn root() -> Markup {
    layout(html! {
        h1 { "Beacon" }
        p {
            "Beacon is a website to track progress for collecting " i { "big fish" } " in Final Fantasy XIV. "
            "To use it, you need an " b { "account ID" } ", which serves to remember which fish you've caught. "
            "If you have an account ID, you can enter it here, or if you don't, create a new one. Usually, your "
            "browser will remember your ID and automatically redirect you, though."
        }

        form method="post" {
            button type="submit" { "Generate a new account ID" }
        }

        p { "or" }

        form method="get" action="/to" {
            input name="id" type="text" placeholder="Enter your account ID...";
            button type="submit" { "Go to tracker" }
        }

        h2 { "What is this about?" }
        p {
            "In Final Fantasy XIV, you can catch a number of elusive and rare fish called big fish. These can be quite tricky to even find -- most of them have a specific time window and only appear during certain weather patterns. This website helps with keeping track of all of that."
        }

        h2 { "Why not use the other tracker website?" }
        p {
            "By all means! " a href="https://ff14fish.carbuncleplushy.com" { "The original tracker" } " is a great resource, and functionally superior to this website. "
            "I mostly created this because of two reasons:"
        }
        ol {
            li { "Everything is rendered on the server - meaning it doesn't eat away at your system resources (good for laptop players!)" }
            li { "Your caught fish are synchronized across devices, and data doesn't get deleted when you clear your browser data" }
        }
        p {
            "These were the two main reasons why I created this. This website couldn't exist without Carbuncle Plushy's tracker -- it uses the same data "
            "that has been painstakingly compiled by hand! If you can, please support them."
        }
    })
}

pub fn new_account(id: String) -> Markup {
    layout(html! {
        p {
            "Your new account ID has been created. Plase save it in a place you remember, like a password manager. If you lose it, you will be unable to "
            "access your account."
        }

        h1 { (id) }

        a href=(format!("/{}", id)) { "Click here to proceed to the tracker" }
    })
}
